@@ -33,6 +33,18 @@ use self as this;
     )
 }
 
+#[test]
+fn doesnt_complete_qualified() {
+    check(
+        r#"
+struct Foo;
+#[foo::$0]
+use self as this;
+"#,
+        expect![[r#""#]],
+    )
+}
+
 #[test]
 fn inside_nested_attr() {
     check(r#"#[cfg($0)]"#, expect![[]])
@@ -75,7 +87,7 @@ fn attr_on_source_file() {
             at no_implicit_prelude
             at no_main
             at no_std
-            at recursion_limit = …
+            at recursion_limit = "…"
             at type_length_limit = …
             at windows_subsystem = "…"
         "#]],
